@@ -30,7 +30,7 @@ from ironic_python_agent import utils
 LOG = log.getLogger(__name__)
 
 
-BIND_MOUNTS = ('/dev', '/proc', '/run')
+BIND_MOUNTS = ('/dev', '/proc')
 
 
 def _get_partition(device, uuid):
@@ -48,7 +48,8 @@ def _get_partition(device, uuid):
             LOG.warning("Couldn't re-read the partition table "
                         "on device %s", device)
 
-        report = utils.execute('lsblk', '-PbioKNAME,UUID,TYPE', device)[0]
+        lsblk = utils.execute('lsblk', '-PbioKNAME,UUID,PARTUUID,TYPE', device)
+        report = lsblk[0]
         for line in report.split('\n'):
             part = {}
             # Split into KEY=VAL pairs
@@ -63,6 +64,10 @@ def _get_partition(device, uuid):
                 LOG.debug("Partition %(uuid)s found on device "
                           "%(dev)s", {'uuid': uuid, 'dev': device})
                 return '/dev/' + part.get('KNAME')
+            if part.get('PARTUUID') == uuid:
+                LOG.debug("Partition %(uuid)s found on device "
+                          "%(dev)s", {'uuid': uuid, 'dev': device})
+                return '/dev/' + part.get('KNAME')
         else:
             error_msg = ("No partition with UUID %(uuid)s found on "
                          "device %(dev)s" % {'uuid': uuid, 'dev': device})
@@ -76,7 +81,8 @@ def _get_partition(device, uuid):
         raise errors.CommandExecutionError(error_msg)
 
 
-def _install_grub2(device, root_uuid, efi_system_part_uuid=None):
+def _install_grub2(device, root_uuid, efi_system_part_uuid=None,
+                   prep_boot_part_uuid=None):
     """Install GRUB2 bootloader on a given device."""
     LOG.debug("Installing GRUB2 bootloader on device %s", device)
     root_partition = _get_partition(device, uuid=root_uuid)
@@ -92,6 +98,10 @@ def _install_grub2(device, root_uuid, efi_system_part_uuid=None):
             efi_partition = _get_partition(device, uuid=efi_system_part_uuid)
             efi_partition_mount_point = os.path.join(path, "boot/efi")
 
+        # For power we want to install grub directly onto the PreP partition
+        if prep_boot_part_uuid:
+            device = _get_partition(device, uuid=prep_boot_part_uuid)
+
         utils.execute('mount', root_partition, path)
         for fs in BIND_MOUNTS:
             utils.execute('mount', '-o', 'bind', fs, path + fs)
@@ -196,13 +206,17 @@ def _install_grub2(device, root_uuid, efi_system_part_uuid=None):
 class ImageExtension(base.BaseAgentExtension):
 
     @base.sync_command('install_bootloader')
-    def install_bootloader(self, root_uuid, efi_system_part_uuid=None):
+    def install_bootloader(self, root_uuid, efi_system_part_uuid=None,
+                           prep_boot_part_uuid=None):
         """Install the GRUB2 bootloader on the image.
 
         :param root_uuid: The UUID of the root partition.
         :param efi_system_part_uuid: The UUID of the efi system partition.
             To be used only for uefi boot mode.  For uefi boot mode, the
             boot loader will be installed here.
+        :param prep_boot_part_uuid: The UUID of the PReP Boot partition.
+            Used only for booting ppc64* partition images locally. In this
+            scenario the bootloader will be installed here.
         :raises: CommandExecutionError if the installation of the
                  bootloader fails.
         :raises: DeviceNotFound if the root partition is not found.
@@ -212,4 +226,5 @@ class ImageExtension(base.BaseAgentExtension):
         iscsi.clean_up(device)
         _install_grub2(device,
                        root_uuid=root_uuid,
-                       efi_system_part_uuid=efi_system_part_uuid)
+                       efi_system_part_uuid=efi_system_part_uuid,
+                       prep_boot_part_uuid=prep_boot_part_uuid)
